local http = require('socket.http')

function getImages(url)
	local image_list = http.request(url)
	image_list, err = loadstring(image_list)
	if err then
		print('Error: '..err)
	end
	checkLocalImages(image_list())
end

function downloadImage(image_url, image_file)
	print('  Downloading: '..image_url)
	local image_data = http.request(image_url)
	love.filesystem.write(image_file, image_data)
end

function checkLocalImages(image_list)
	if not love.filesystem.exists('submissions') then
		love.filesystem.mkdir('submissions')
	end
	for i=1, #image_list do
		local author = image_list[i].author
		print(author)
		if not love.filesystem.exists('submissions/'..author) then
			love.filesystem.mkdir('submissions/'..author)
		end
		for j=1, #image_list[i] do
			local image_file = string.match(image_list[i][j], '/([%w_]+.png)$')
			if not love.filesystem.exists('submissions/'..author..'/'..image_file) then
				local image_url = image_list[i][j]
				image_data = downloadImage(image_url, 'submissions/'..author..'/'..image_file)
			else
				print('  File already exists.')
			end
		end
	end
end

function loadImages()
	local imageTable = {}
	local authors = love.filesystem.enumerate('submissions')
	for _,author in ipairs(authors) do
		print(author)
		local authorDir = 'submissions/'..author..'/'
		local images = love.filesystem.enumerate(authorDir)
		for _,title in ipairs(images) do
			print('  '..title)
			imageDir = authorDir..title
			if love.filesystem.isFile(imageDir) then
				local fileName = string.lower(imageDir)
				if string.find(fileName, '.png$') then
					local image, width, height = createImage(imageDir)
					local imageEntry = {
							image = image,
							author = author,
							title = title,
							width = width,
							height = height}
					table.insert(imageTable, imageEntry)
				end
			end
		end
	end
	return imageTable
end

function newPaddedImage(filename)
    local source = love.image.newImageData(filename)
    local w, h = source:getWidth(), source:getHeight()
    
    -- Find closest power-of-two.
    local wp = math.pow(2, math.ceil(math.log(w)/math.log(2)))
    local hp = math.pow(2, math.ceil(math.log(h)/math.log(2)))
    
    -- Only pad if needed:
    if wp ~= w or hp ~= h then
        local padded = love.image.newImageData(wp, hp)
        padded:paste(source, 0, 0)
        return love.graphics.newImage(padded)
    end
    
    return love.graphics.newImage(source)
end

function createImage(imageDir)
	--TODO: PO2!!!
	local image_data = love.image.newImageData(imageDir)
	local width, height = image_data:getWidth(), image_data:getHeight()
	local image = newPaddedImage(imageDir)
	image:setFilter('nearest','nearest')
	return image, width, height
end

function love.keypressed(k)
	if k==' ' then
		blackBackground = (blackBackground + 1) % 3
	end
end

function love.load()
	--image_table = getImages('http://dl.dropbox.com/u/559759/PixelShow/images.lua')
	blackBackground = 2
	love.graphics.setFont(12)
	images = loadImages()
	width_sum = 0
	for i=1,#images do
		width_sum = width_sum + images[i].width*16+32
	end
	x_offset = 0
	mouse_down = false
	mouse_scroll = false
end

function love.update(dt)
	if love.keyboard.isDown('right') then
		x_offset = x_offset - dt*500
	elseif love.keyboard.isDown('left') then
		x_offset = x_offset + dt*500
	end

	if mouse_down then
		local m_x = love.mouse.getX()
		x_offset = x_offset + m_x - mouse_x
		mouse_scroll = (m_x - mouse_x) / dt
		mouse_x = m_x
	elseif mouse_scroll then
		x_offset = x_offset + mouse_scroll * dt
		mouse_scroll = mouse_scroll / 10 ^ dt
		if math.abs(mouse_scroll) < 1 then
			mouse_scroll = false
		end
	end

	if x_offset < -width_sum+768 then
		x_offset = -width_sum+768
	elseif x_offset > 0 then
		x_offset = 0
	end
end

function love.draw()
	if blackBackground == 0 then
		love.graphics.setBackgroundColor(0, 0, 0)
	elseif blackBackground == 1 then
		love.graphics.setBackgroundColor(255, 255, 255)
	else
		love.graphics.setBackgroundColor(80, 80, 80)
	end
	local width_sum = 0
	love.graphics.push()
	love.graphics.translate(math.floor(x_offset), 0)
	for i=1,#images do
		love.graphics.push()
		love.graphics.translate(width_sum, 0)
		if blackBackground ~= 1 then
			love.graphics.setColor(255, 255, 255)
		else
			love.graphics.setColor(0, 0, 0)
		end
		love.graphics.print(images[i].author,0,0)
		love.graphics.print(images[i].title,0,14)
		love.graphics.setColor(255,255,255)
		love.graphics.draw(images[i].image, 0, 28, 0, 16)
		width_sum = width_sum + images[i].width*16+32
		love.graphics.pop()
	end
	love.graphics.pop()
end

function love.mousepressed(x)
	mouse_down = true
	mouse_x = x
end

function love.mousereleased()
	mouse_down = false
end
